'use strict';

angular.module('raml', [])
    .factory('ramlReader', function () {
        return {
            processBaseUri: function (definition) {
                var baseUri = (definition.baseUri || '').replace(/\/\/*$/g, '');
                var version = definition.version || '';

                baseUri = baseUri.replace(':0', '\\:0');
                baseUri = baseUri.replace(':1', '\\:1');
                baseUri = baseUri.replace(':2', '\\:2');
                baseUri = baseUri.replace(':3', '\\:3');
                baseUri = baseUri.replace(':4', '\\:4');
                baseUri = baseUri.replace(':5', '\\:5');
                baseUri = baseUri.replace(':6', '\\:6');
                baseUri = baseUri.replace(':7', '\\:7');
                baseUri = baseUri.replace(':8', '\\:8');
                baseUri = baseUri.replace(':9', '\\:9');

                return baseUri.replace('{version}', version);
            },
            readRootElements: function (raml) {
                var result = {};

                if (typeof raml.title !== 'undefined') {
                    result.title = raml.title;
                } else {
                    throw new Error('title is not defined');
                }

                if (typeof raml.baseUri !== 'undefined') {
                    result.baseUri = raml.baseUri;
                } else {
                    throw new Error('baseUri is not defined');
                }


                if (typeof raml.version !== 'undefined') {
                    result.version = raml.version;
                }

                return result;
            },
            readDocumentation: function (raml) {
                var result = {};

                if (typeof raml.documentation !== 'undefined') {
                    result.documentation = raml.documentation;
                }

                return result;
            },
            convert: function (query) {
                var queryParams = [];
                var param;

                for (var prop in query) {
                    param = query[prop];
                    param.paramName = prop;
                    queryParams.push(param);
                }

                return queryParams;
            },
            readHttpMethodData: function (methodDescriptor) {
                var result = {};

                if (typeof methodDescriptor.method !== 'undefined') {
                    result.name = methodDescriptor.method;
                }

                if (typeof methodDescriptor.summary !== 'undefined') {
                    result.summary = methodDescriptor.summary;
                }

                if (typeof methodDescriptor.responses !== 'undefined') {
                    result.responses = methodDescriptor.responses;

                    for (var prop in result.responses) {
                        if (result.responses[prop] === null) {
                            result.responses[prop] = prop;
                        }
                    }
                }

                if (typeof methodDescriptor.queryParameters !== 'undefined') {
                    result.queryParameters = [];

                    angular.forEach(methodDescriptor.queryParameters, function () {
                        result.queryParameters = this.convert(methodDescriptor.queryParameters);
                    }.bind(this));
                }

                if (typeof methodDescriptor.uriParameters !== 'undefined') {
                    result.uriParameters = methodDescriptor.uriParameters;
                }

                if (typeof methodDescriptor.body !== 'undefined') {
                    result.request = methodDescriptor.body;

                    for (var contentType in result.request) {
                        if (typeof result.request[contentType].formParameters !== 'undefined') {
                            for (var param in result.request[contentType].formParameters) {
                                var temp = JSON.parse(JSON.stringify(result.request[contentType].formParameters[param]));

                                result.request[contentType].formParameters[param].paramName = param;
                                temp.paramName = param;

                                if (temp.type === 'file') {
                                    delete result.request[contentType].formParameters[param];

                                    if (typeof result.request[contentType].formParameters.__files === 'undefined') {
                                        result.request[contentType].formParameters.__files = {};
                                    }

                                    result.request[contentType].formParameters.__files[param] = temp;
                                }
                            }
                        }
                    }
                }

                return result;
            },
            readContentTypes: function (methodDescriptor) {
                var types = [];

                if (typeof methodDescriptor.body !== 'undefined') {
                    for (var type in methodDescriptor.body) {
                        if (types.indexOf(type) === -1) {
                            types.push(type);
                        }
                    }
                }

                // if (typeof methodDescriptor.responses !== 'undefined') {
                //     angular.forEach(methodDescriptor.responses, function (element) {
                //         for (var httpCode in element) {
                //             for (var contentType in methodDescriptor.responses[httpCode]) {
                //                 if (types.indexOf(contentType) === -1) {
                //                     types.push(contentType);
                //                 }
                //             }
                //         }
                //     });
                // }

                return types;
            },
            readTraits: function (traitList, traitsDescription) {
                var traits = [];

                traitList.forEach(function (use) {
                    var traitName, found;

                    if (typeof use === 'string') {
                        traitName = use;
                    } else if (typeof use === 'object') {
                        for (var key in use) {
                            if (use.hasOwnProperty(key)) {
                                traitName = key;
                            }
                        }
                    }

                    if (traitName) {
                        found = traitsDescription
                            .filter(function (t) {
                                return t[traitName];
                            })
                            .map(function (t) {
                                return t[traitName];
                            });

                        traits.push.apply(traits, found);
                    }
                });

                return traits;
            },
            readResourceData: function (resource, raml) {
                var result = JSON.parse(JSON.stringify(resource));

                if (!resource.methods instanceof Array) {
                    delete result.methods;
                }

                if (typeof result.name === 'undefined') {
                    result.name = result.relativeUri;
                }

                if (typeof result.relativeUri === 'undefined') {
                    throw new Error('relativeUri is not defined');
                }

                if (typeof resource.methods !== 'undefined') {
                    if (resource.methods instanceof Array) {
                        result.methods = {};
                        angular.forEach(resource.methods, function (element) {
                            result.methods[element.method] = this.readHttpMethodData(element);
                            result.methods[element.method].supportedTypes = this.readContentTypes(element);
                        }.bind(this));
                    }
                }

                if (result.traits) {
                    result.traits.concat(this.readTraitsDeep(resource, raml.traits));
                } else {
                    result.traits = this.readTraitsDeep(resource, raml.traits);
                }

                result.absoluteUri = raml.baseUri + result.relativeUri;

                return result;
            },
            readTraitsDeep: function (resource, traitsDetails) {
                var traits = [];

                if (typeof resource.is !== 'undefined') {
                    traits = this.readTraits(resource.is, traitsDetails);
                }

                angular.forEach(resource.methods, function (method) {
                    if (typeof method.is !== 'undefined') {
                        traits = traits.concat(this.readTraits(method.is, traitsDetails));
                    }
                }.bind(this));

                return traits;
            },
            readRootResources: function (raml) {
                var result = {
                    resources: []
                };

                if (typeof raml.resources !== 'undefined') {
                    angular.forEach(raml.resources, function (element) {
                        result.resources.push(this.readResourceData(element, raml));
                    }.bind(this));
                }

                return result;
            },
            read: function (raml) {
                var rootResources = this.readRootResources(raml),
                    rootDocumentation = this.readDocumentation(raml),
                    result;

                angular.forEach(rootResources.resources, function (resource) {
                    var flatResources = this.flatten(resource);

                    delete resource.resources;

                    resource.resources = [];

                    angular.forEach(flatResources, function (el) {
                        var r = this.readResourceData(el, raml);
                        resource.resources.push(r);
                    }.bind(this));
                }.bind(this));

                result = this.readRootElements(raml);
                result.documentation = rootDocumentation.documentation;
                result.resources = rootResources.resources;

                return result;
            },
            flatten: function (resource, container) {
                var result = [],
                    temp, uriPart = resource.relativeUri;

                if (typeof container === 'undefined') {
                    temp = JSON.parse(JSON.stringify(resource));

                    delete temp.resources;

                    if (typeof temp.methods !== 'undefined') {
                        result = [temp];
                    }

                } else {
                    result = container;
                }

                if (typeof resource.resources === 'undefined') {
                    resource.resources = [];
                }

                if (resource.resources.length > 0) {
                    angular.forEach(resource.resources, function (el) {
                        temp = JSON.parse(JSON.stringify(el));

                        delete temp.resources;

                        temp.relativeUri = uriPart + temp.relativeUri;
                        el.relativeUri = temp.relativeUri;

                        result.push(temp);

                        return this.flatten(el, result);
                    }.bind(this));

                    return result;
                } else {
                    return result;
                }
            }
        };
    });